#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                $Date$
#                $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
import sys

#Maximum size of byte.
_MAX_BYTE_SIZE = 0xFF
_NIBBLE = 4
#Value of Hex A in decimal.
_HEX_A_DECIMAL_VALUE = 10

###Python 2 requires this
#pylint: disable=bad-option-value,old-style-class
class GXCommon:
    #pylint: disable=too-few-public-methods
    """General methods for communication."""

    def __init__(self, data=None, senderInfo=None):
        """
        Constructor.
        """

    @classmethod
    def getVersion(cls):
        """Get version."""
        return sys.version_info

    @classmethod
    def toHex(cls, value):
        """Convert data to hex."""
        #Return empty string if array is empty.
        if not value:
            return ""
        __hexArray = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F')
        LOW_BYTE_PART = 0x0F
        hexChars = ""
        for it in value:
            hexChars += __hexArray[it >> _NIBBLE]
            hexChars += __hexArray[it & LOW_BYTE_PART]
            hexChars += ' '
        return hexChars

    #Convert char hex value to byte value.
    @classmethod
    def ___getValue(cls, c):
        #Id char.
        if c > '9':
            if c > 'Z':
                value = c.encode()[0] - 'a'.encode()[0]
            else:
                value = c.encode()[0] - 'A'.encode()[0]
            value += _HEX_A_DECIMAL_VALUE
        else:
            #If number.
            value = c.encode()[0] - '0'.encode()[0]
        return value

    @classmethod
    def hexToBytes(cls, value):
        ###Convert string to byte array.###
        buff = bytearray()
        lastValue = -1
        for ch in value:
            if '0' >= ch < 'g':
                if lastValue == -1:
                    lastValue = cls.___getValue(ch)
                elif lastValue != -1:
                    buff.append(lastValue << _NIBBLE | cls.___getValue(ch))
                    lastValue = -1
            elif lastValue != -1:
                buff.append(cls.___getValue(ch))
                lastValue = -1
        return buff



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                $Date$
#                $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------

###Python 2 requires this ot property is not working.
#pylint: disable=bad-option-value, useless-object-inheritance
class ReceiveParameters(object):
    """ReceiveParameters class is used when data is read synchronously."""
    #pylint: disable=too-few-public-methods

    def __init__(self):
        """Constructor."""
        self.__waitTime = -1
        ###How long reply message is waited.
        self.allData = False
        ###Is all data read when End of packet is found.
        self.reply = None
        ###Received data.
        self.peek = False
        ###If true, returns the bytes from the buffer without removing.
        self.__eop = None
        ###End of packet waited for.
        self.__count = 0
        ###Minimum count of bytes waited for.


    def __getEop(self):
        return self.__eop

    def __setEop(self, value):
        self.__eop = value

    eop = property(__getEop, __setEop)
    """The end of packet (EOP) waited for."""

    def __getCount(self):
        return self.__count

    def __setCount(self, value):
        if value < 0:
            raise ValueError("Count")
        self.__count = value

    count = property(__getCount, __setCount)
    """The number of reply data bytes to be read."""

    def __getWaitTime(self):
        return self.__waitTime

    def __setWaitTime(self, value):
        self.__waitTime = value

    waitTime = property(__getWaitTime, __setWaitTime)
    """Maximum time, in milliseconds, to wait for reply data. WaitTime -1
    (Default value) indicates infinite wait time."""



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                $Date$
#                $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
import datetime
from .GXCommon import GXCommon

###Python 2 requires this ot property is not working.
#pylint: disable=bad-option-value, useless-object-inheritance
class TraceEventArgs(object):
    #pylint: disable=too-few-public-methods
    """Argument class for IGXMedia connection and disconnection events."""

    def __init__(self, traceType, data, index=0, length=None):
        """
        Constructor.

        type : Trace type.
        data : Send or received data.
        index : Index where data copy is started.
        length : How many bytes are included to data.
        """

        ###Time stamp when data is send or received.
        self.timestamp = datetime.datetime.now().time()
        ###Trace type.
        self.type = traceType
        if isinstance(data, (bytearray, bytes)):
            if length is None or index != 0:
                length = len(data) - index
            ###Send or received data.
            self.__data = data[index:length]
        else:
            self.__data = data

    def __getData(self):
        return self.__data

    def __setData(self, value):
        self.__data = value

    data = property(__getData, __setData)
    """Send or received data."""

    def dataToString(self):
        """Convert data to string."""
        if not self.data:
            return ""
        if isinstance(self.data, (bytearray, bytes)):
            return GXCommon.toHex(self.data)
        return str(self.data)

    def __str__(self):
        return self.timestamp.strftime("%H:%M:%S") + "\t" + str(self.type) + "\t" + self.dataToString()



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                $Date$
#                $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
import abc
from .GXCommon import GXCommon

ABC = abc.ABCMeta('ABC', (object,), {'__slots__': ()})
#pylint: disable=syntax-error
if GXCommon.getVersion() >= (2, 7):
    class IGXMedia(ABC):
        #pylint: disable=too-many-public-methods
        """Common interface for all Media components.
        Using this interface GXCommunication library enables communication with\
        different medias."""

        @abc.abstractmethod
        def addListener(self, listener):
            """Start to listen media events."""

        @abc.abstractmethod
        def removeListener(self, listener):
            """Stop to listen media events."""

        @abc.abstractmethod
        def copy(self, target):
            """ Copies the content of the media to target media.
            target : Target media."""

        @abc.abstractmethod
        def getName(self):
            """ Returns name of the media. Media name is used to identify media
            connection, so two different media connection can not return same media
            name.

            Returns Media name."""

        @abc.abstractproperty
        def trace(self):
            """Trace level of the IGXMedia."""

        @abc.abstractmethod
        def open(self):
            """Opens the media."""

        @abc.abstractmethod
        def isOpen(self):
            """Checks if the connection is established.
            Returns True, if the connection is established."""

        @abc.abstractmethod
        def close(self):
            """Closes the active connection."""

        @abc.abstractmethod
        def send(self, data, receiver):
            """Sends data asynchronously. No reply from the receiver, whether or not the
            operation was successful, is expected.

            data : Data to send to the device.
            receiver : Media depend information of the receiver (optional)."""

        @abc.abstractmethod
        def getMediaType(self):
            """Returns media type as a string."""

        @abc.abstractmethod
        def getSettings(self):
            """Get media settings as a XML string."""

        @abc.abstractmethod
        def setSettings(self, value):
            """Set media settings as a XML string."""

        @abc.abstractmethod
        def getSynchronous(self):
            """Locking this property makes the connection synchronized and stops sending
            OnReceived events."""

        @abc.abstractmethod
        def getIsSynchronous(self):
            """Checks if the connection is in synchronous mode."""

        @abc.abstractmethod
        def receive(self, args):
            """Waits for more reply data After SendSync if whole packet is not received yet."""

        @abc.abstractmethod
        def resetSynchronousBuffer(self):
            """Resets synchronous buffer."""

        @abc.abstractmethod
        def getBytesSent(self):
            """Sent byte count."""

        @abc.abstractmethod
        def getBytesReceived(self):
            """Received byte count."""

        @abc.abstractmethod
        def resetByteCounters(self):
            """Resets BytesReceived and BytesSent counters."""

        @abc.abstractmethod
        def validate(self):
            """Validate Media settings for connection open. Returns table of media
            properties that must be set before media is valid to open."""

        @abc.abstractproperty
        def eop(self):
            """End of the packet."""
else:
    class IGXMedia(ABC):
        #pylint: disable=too-many-public-methods
        """Common interface for all Media components.
        Using this interface GXCommunication library enables communication with\
        different medias."""

        @abc.abstractmethod
        def addListener(self, listener):
            """Start to listen media events."""

        @abc.abstractmethod
        def removeListener(self, listener):
            """Stop to listen media events."""

        @abc.abstractmethod
        def copy(self, target):
            """ Copies the content of the media to target media.
            target : Target media."""

        @abc.abstractmethod
        def getName(self):
            """ Returns name of the media. Media name is used to identify media
            connection, so two different media connection can not return same media
            name.

            Returns Media name."""

        @abc.abstractproperty
        def trace(self):
            """Trace level of the IGXMedia."""

        @abc.abstractmethod
        def open(self):
            """Opens the media."""

        @abc.abstractmethod
        def isOpen(self):
            """Checks if the connection is established.
            Returns True, if the connection is established."""

        @abc.abstractmethod
        def close(self):
            """Closes the active connection."""

        @abc.abstractmethod
        def send(self, data, receiver):
            """Sends data asynchronously. No reply from the receiver, whether or not the
            operation was successful, is expected.

            data : Data to send to the device.
            receiver : Media depend information of the receiver (optional)."""

        @abc.abstractmethod
        def getMediaType(self):
            """Returns media type as a string."""

        @abc.abstractmethod
        def getSettings(self):
            """Get media settings as a XML string."""

        @abc.abstractmethod
        def setSettings(self, value):
            """Set media settings as a XML string."""

        @abc.abstractmethod
        def getSynchronous(self):
            """Locking this property makes the connection synchronized and stops sending
            OnReceived events."""

        @abc.abstractmethod
        def getIsSynchronous(self):
            """Checks if the connection is in synchronous mode."""

        @abc.abstractmethod
        def receive(self, args):
            """Waits for more reply data After SendSync if whole packet is not received yet."""

        @abc.abstractmethod
        def resetSynchronousBuffer(self):
            """Resets synchronous buffer."""

        @abc.abstractmethod
        def getBytesSent(self):
            """Sent byte count."""

        @abc.abstractmethod
        def getBytesReceived(self):
            """Received byte count."""

        @abc.abstractmethod
        def resetByteCounters(self):
            """Resets BytesReceived and BytesSent counters."""

        @abc.abstractmethod
        def validate(self):
            """Validate Media settings for connection open. Returns table of media
            properties that must be set before media is valid to open."""

        @abc.abstractproperty
        def eop(self):
            """End of the packet."""



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                $Date$
#                $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
import abc
ABC = abc.ABCMeta('ABC', (object,), {'__slots__': ()})
class IGXMediaListener(ABC):
    #pylint: disable=no-self-argument
    __metaclass__ = abc.ABCMeta
    """Media component will notify events throught this interface."""

    @abc.abstractmethod
    def onError(sender, ex):
        """
        Represents the method that will handle the error event of a Gurux
        component.

        sender :  The source of the event.
        ex : An Exception object that contains the event data.
        """

    @abc.abstractmethod
    def onReceived(sender, e):
        """Media component sends received data through this method.

        sender : The source of the event.
        e : Event arguments.
        """

    @abc.abstractmethod
    def onMediaStateChange(sender, e):
        """Media component sends notification, when its state changes.
        sender : The source of the event.
        e : Event arguments.
        """

    @abc.abstractmethod
    def onTrace(sender, e):
        """Called when the Media is sending or receiving data.

        sender : The source of the event.
        e : Event arguments.
        """

    @abc.abstractmethod
    def onPropertyChanged(sender, e):
        """
        Event is raised when a property is changed on a component.

        sender : The source of the event.
        e : Event arguments.
        """
